package org.embl.gbcs.jemultiplexer;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.zip.GZIPOutputStream;

import net.sf.picard.fastq.AsyncFastqWriter;
import net.sf.picard.fastq.BasicFastqWriter;
import net.sf.picard.fastq.FastqWriter;
import net.sf.picard.fastq.FastqWriterFactory;
import net.sf.samtools.Defaults;
import net.sf.samtools.util.Md5CalculatingOutputStream;

public class JemultiplexerFastqWriterFactory {

	boolean useAsyncIo = Defaults.USE_ASYNC_IO;
	

	/** Sets whether or not to use async io (i.e. a dedicated thread per writer. */
    public void setUseAsyncIo(final boolean useAsyncIo) { this.useAsyncIo = useAsyncIo; }

	
	public FastqWriter newWriter(final File out, boolean gzip, boolean createMd5) {
		
		if(!gzip)
			return newWriter(out, createMd5);
					
					
		GZIPOutputStream go;
		try {
			if(createMd5)
				go = new GZIPOutputStream(
						new Md5CalculatingOutputStream(
								new FileOutputStream(out),
								new File(out.getAbsolutePath() + ".md5"))
						);
			else
				go = new GZIPOutputStream(new FileOutputStream(out));
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
        final FastqWriter writer = new BasicFastqWriter( new PrintStream(go) );
        if (useAsyncIo) {
            return new AsyncFastqWriter(writer, AsyncFastqWriter.DEFAULT_QUEUE_SIZE);
        }
        else {
            return writer;
        }
    }
	
	
	public FastqWriter newWriter(final File out, boolean createMd5) {
		if(!createMd5)
			return newWriter(out);
		
		
		Md5CalculatingOutputStream go;
		try {
			go =new Md5CalculatingOutputStream(
					new FileOutputStream(out),
					new File(out.getAbsolutePath() + ".md5"));

		} catch (IOException e) {
			throw new RuntimeException(e);
		}
        final FastqWriter writer = new BasicFastqWriter( new PrintStream(go) );
	    if (useAsyncIo) {
	        return new AsyncFastqWriter(writer, AsyncFastqWriter.DEFAULT_QUEUE_SIZE);
	    }
	    else {
	        return writer;
	    }
	}
	
	public FastqWriter newWriter(final File out) {
	    final FastqWriter writer = new BasicFastqWriter(out);
	    if (useAsyncIo) {
	        return new AsyncFastqWriter(writer, AsyncFastqWriter.DEFAULT_QUEUE_SIZE);
	    }
	    else {
	        return writer;
	    }
	}
	
}
